----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    11:27:57 09/07/2016 
-- Design Name: 
-- Module Name:    fbin2bcd - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity fbin2bcd is
	 Generic ( Mf : integer); -- num of bits
    Port ( clk100mhz : in  STD_LOGIC;
           fdata2conv : in  STD_LOGIC_VECTOR (6 downto 0); 	  -- binary number to be converted
           conv_dataf : out  STD_LOGIC_VECTOR (27 downto 0)); -- conversion results

end fbin2bcd;

architecture Behavioral of fbin2bcd is
-- State machine for binary to BCD conversion of fractional part
type state_type is (idle, shift, check, adjust, done);
signal state : state_type := idle;
-- Shift register counter
signal count : integer range 0 to Mf := Mf;
-- BCD registers for 7 SSD's
signal bcd0, bcd1, bcd2, bcd3, bcd4, bcd5, bcd6 : unsigned (3 downto 0) := (others => '0');
-- shift register
signal temp_data : std_logic_vector (6 downto 0) := (others => '0');

begin

	f_conversion: process (clk100mhz)
	begin
		if (rising_edge(clk100mhz)) then
		case state is
		
		-- load data to be converted into shift register and clean bcd registers
		when idle =>
			count <= Mf;
			temp_data <= fdata2conv;
			bcd0 <= x"0";
			bcd1 <= x"0";
			bcd2 <= x"0";
			bcd3 <= x"0";
			bcd4 <= x"0";
			bcd5 <= x"0";
			bcd6 <= x"0";
			state <= shift;
		
		-- shift operation (right shift) from shift register to bcd registers is performed here
		when shift =>
			bcd6 <= bcd5(0) & bcd6(3 downto 1);
			bcd5 <= bcd4(0) & bcd5(3 downto 1);
			bcd4 <= bcd3(0) & bcd4(3 downto 1);
			bcd3 <= bcd2(0) & bcd3(3 downto 1);
			bcd2 <= bcd1(0) & bcd2(3 downto 1);
			bcd1 <= bcd0(0) & bcd1(3 downto 1);
			bcd0 <= temp_data(0)& bcd0(3 downto 1);
			temp_data <= '0' & temp_data(6 downto 1);
			if (count = 0) then
				state <= check;
			else
				count <= count-1;
				state <= check;
			end if;
		
		-- this state checks whether the new content of any bcd register needs to be adjusted or not
		when check =>
			if (bcd6 > 7 or bcd5 > 7 or bcd4 > 7 or bcd3 > 7 or bcd2 > 7 
			or bcd1 > 7 or bcd0 > 7) then
				state <= adjust;
			else
				if (count > 0) then
					state <= shift;
				else
					state <= done;
				end if;
			end if;
		
		-- this state handles adjustment of bcd registers' values when necessary
		when adjust =>
			if (bcd0 > 7) then
				bcd0 <= bcd0+13;
			end if;
			if (bcd1 > 7) then
				bcd1 <= bcd1+13;
			end if;
			if (bcd2 > 7) then
				bcd2 <= bcd2+13;
			end if;
			if (bcd3 > 7) then
				bcd3 <= bcd3+13;
			end if;
			if (bcd4 > 7) then
				bcd4 <= bcd4+13;
			end if;
			if (bcd5 > 7) then
				bcd5 <= bcd5+13;
			end if;
			if (bcd6 > 7) then
				bcd6 <= bcd6+13;
			end if;
			if (count = 0) then
				state <= done;
			else
				state <= shift;
			end if;	
		
		-- update module's output with the new conversion results
		when done =>
			conv_dataf <= std_logic_vector(bcd0)&std_logic_vector(bcd1)&
			std_logic_vector(bcd2)&std_logic_vector(bcd3)&std_logic_vector(bcd4)
			&std_logic_vector(bcd5)&std_logic_vector(bcd6);
			count <= 0;
			state <= idle;
			
		end case;	
		end if;
	end process f_conversion;

end Behavioral;

