close all;
clear all;
clc;

% generate signal to drive FuncGen with
N=2^14;
VDAC=2.5;
VADC=2.5;
Nbit=2^12;
dv_dac=VDAC/Nbit;
dv_adc=VADC/Nbit;
n=0:1:N-1;
% sig=uint16(((Nbit/2-1/3))*(square(0.05*n)+1));
% sig=uint16(N/2*(square(0.05*n)+1));
% sig=uint16(N/2*(sin(0.05*n)+1)-1);
% sig=uint16(N/2*(sawtooth(0.05*n)+1));
sig=zeros(1,N);
for j=1:N
    if (j<=Nbit)
        sig(1,j)=j-1;
    elseif (j<=2*Nbit && j>Nbit)
        sig(1,j)=j-1-Nbit; 
    elseif (j<=3*Nbit && j>2*Nbit)
        sig(1,j)=j-1-2*Nbit;
    else
        sig(1,j)=j-1-3*Nbit;
    end
end
% plot(n,sig)
sig_fb=zeros(1,length(sig)); % array to store feedback signal

DBR =[9600 14400 19200 38400 57600 115200 230400 460800 921600]; % last one not recommended

% set and open serial port
s=serial('COM5');
set(s,'BaudRate',DBR(8),'Parity','none','StopBits',1,'ByteOrder','littleEndian')
set(s,'OutputBufferSize',2);
set(s,'InputBufferSize',2);
s.FlowControl='none'; 
fopen(s);

% run FuncGen
tic
for j=1:N
    % write to fpga
     while(strcmp(s.Pinstatus.ClearToSend,'off')) % wait until FuncGen is ready to receive next command
         pause(1e-7);
     end;
    fwrite(s,sig(j),'uint16');
    % read results from fpga
    sig_fb(1,j)=fread(s,1,'uint16');
end
time=toc % to check speed of FuncGen
s.ValuesReceived % check amount of data received from FuncGen
frequency=N/time % frequency of FuncGen operation (number of full loops (write/read FPGA) per second)
% close and delete port
fclose(s);
delete (s);
clear s;

%% Check the results
set(0,'defaultaxesfontsize',12)
set(0,'defaultlinelinewidth',1.0)
delta=zeros(1,N-1);delta_v=zeros(1,N-1);
for j=1:N
    delta_v(j)=dv_dac*sig(j)-dv_adc*sig_fb(j);
    delta(j)=sig(j)-sig_fb(j);
end
figure(1)
plot(n,dv_dac*sig,'b',n,dv_adc*sig_fb,'r');axis tight
% plot(n,sig,'b',n,sig_fb,'r')
xlabel('n, sample')
ylabel('Amplitude, V')
legend('Tx','Rx')
title('Function Generator (Tx-DAC-ADC-RX)')
figure(2)
plot(n,delta_v,'b');axis tight;
xlabel('n, sample')
ylabel('Amplitude Error, V')
title('Error between Rx and Tx (DAC and ADC)')
figure(3)
plot(n(1:4096),delta_v(1:4096),'b',n(1:4096),delta_v(4097:8192),'--r',...
    n(1:4096),delta_v(8193:12288),'-.g')%,n(1:4096),delta_v(12289:16384),'.k');
axis tight;
title ('Error overlay for the different cycles of the saw-tooth wave')
xlabel('n, sample')
ylabel('Amplitude Error, V')
