----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    10:49:08 09/06/2016 
-- Design Name: 
-- Module Name:    ibin2bcd - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity ibin2bcd is
	 Generic ( Mi : integer); -- num of bits
    Port ( clk100mhz : in  STD_LOGIC;
           idata2conv : in  STD_LOGIC_VECTOR (3 downto 0);   -- binary number to be converted
           conv_datai : out  STD_LOGIC_VECTOR (3 downto 0)); -- conversion results
end ibin2bcd;

architecture Behavioral of ibin2bcd is
-- State machina for binary-to-BCD converter for integer part
type state_type is (idle, shift, check, adjust, done);
signal state : state_type := idle;
-- Counter to control shift operation
signal count : integer range 0 to Mi-1 := Mi-1;
-- Register to store BCD
signal bcd0 : unsigned (3 downto 0) := (others => '0');
-- Shift Register to store binary number to be converted
signal temp_data : std_logic_vector (3 downto 0) := (others => '0');

begin
	i_conversion: process (clk100mhz)
	begin
		if (rising_edge(clk100mhz)) then
		case state is
		
		-- load data to be converted into shift register
		when idle =>
			count <= Mi-1;
			temp_data <= idata2conv; -- load input to shift register
			bcd0 <= x"0"; -- clear bcd register
			state <= shift;
		
		-- shift operation (left shift) from shift register to bcd register is performed here
		when shift =>
			bcd0 <= bcd0(2 downto 0) & temp_data(3);
			temp_data <= temp_data(2 downto 0) & '0';
			count <= count-1;
			if (count = 0) then
				state <= done;
			else
				state <= check;
			end if;
			
		-- this state checks whether the new content of bcd register needs to be adjusted or not
		when check =>
			if (bcd0 > 4) then
				state <= adjust;
			else
				state <= shift;
			end if;
		
		-- this state handles adjustment of bcd register's value when necessary
		when adjust =>
			if (bcd0 > 4) then
				bcd0 <= bcd0+3;
			end if;
			state <= shift;
		
		-- update module's output with the new conversion results
		when done =>
			conv_datai <= std_logic_vector(bcd0);
			count <= 0;
			state <= idle;
			
		end case;	
		
		end if;
	end process i_conversion;

end Behavioral;

